'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const client = require('react-dom/client');
const testUtils = require('react-dom/test-utils');
const environment$1 = require('../environment.cjs');
const react = require('./shared/react.cjs');

const environment = environment$1.createEnvironment({
  act: testUtils.act,
  mount(tree) {
    const element = document.createElement('div');
    document.body.appendChild(element);
    const root = client.createRoot(element);
    root.render(tree);
    return {
      root,
      element
    };
  },
  unmount({
    root,
    element
  }) {
    root.unmount();
    element.remove();
  },
  update(instance, create) {
    return createNodeFromFiber(instance._reactInternals, create);
  }
});
const {
  render,
  createRender,
  rendered,
  destroyAll
} = environment;
function createNodeFromFiber(element, create) {
  const fiber = react.findCurrentFiberUsingSlowPath(element);
  if (fiber.tag === react.Tag.HostText) {
    return fiber.memoizedProps;
  }
  const props = {
    ...(fiber.memoizedProps || {})
  };
  const instance = fiber.stateNode;
  const isDOM = typeof fiber.type === 'string';
  function children() {
    let currentFiber = fiber.child;
    const children = [];
    while (currentFiber != null) {
      const result = createNodeFromFiber(currentFiber, create);
      children.push(result);
      currentFiber = currentFiber.sibling;
    }
    return children;
  }
  return create({
    props,
    children,
    type: fiber.type,
    instance: fiber.stateNode,
    get isDOM() {
      return isDOM;
    },
    get domNodes() {
      return getDomNodes(this);
    },
    get domNode() {
      const domNodes = getDomNodes(this);
      if (domNodes.length > 1) {
        throw new Error('You can’t call getDOMNode() on an element that returns multiple HTML elements. Call getDOMNodes() to retrieve all of the elements instead.');
      }
      return domNodes[0] ?? null;
    },
    get html() {
      if (instance instanceof HTMLElement) {
        return instance.outerHTML;
      }
      return this.children.reduce((text, child) => `${text}${typeof child === 'string' ? child : child.html}`, '');
    },
    get text() {
      if (instance instanceof HTMLElement) return instance.textContent ?? '';
      return this.children.reduce((text, child) => `${text}${typeof child === 'string' ? child : child.text}`, '');
    },
    data(key) {
      return props[key.startsWith('data-') ? key : `data-${key}`];
    }
  });
  function getDomNodes(node) {
    if (isDOM) return [instance];
    return node.children.filter(child => typeof child !== 'string' && child.isDOM).map(child => child.instance);
  }
}

exports.isNode = environment$1.isNode;
exports.createNodeFromFiber = createNodeFromFiber;
exports.createRender = createRender;
exports.destroyAll = destroyAll;
exports.environment = environment;
exports.render = render;
exports.rendered = rendered;
