'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const reactTestRenderer = require('react-test-renderer');
const environment$1 = require('../environment.cjs');

const environment = environment$1.createEnvironment({
  act: reactTestRenderer.act,
  mount(tree) {
    const renderer = reactTestRenderer.create(tree);
    return {
      renderer
    };
  },
  unmount({
    renderer
  }) {
    renderer.unmount();
  },
  update(_, create, {
    renderer
  }) {
    return createNodeFromTestInstance(renderer.root, create);
  }
});
const {
  render,
  createRender,
  rendered,
  destroyAll
} = environment;
function createNodeFromTestInstance(testInstance, create) {
  function children() {
    return testInstance.children.map(child => typeof child === 'string' ? child : createNodeFromTestInstance(child, create)) ?? [];
  }
  return create({
    props: testInstance.props,
    type: testInstance.type,
    children,
    instance: testInstance.instance
  });
}

exports.createRender = createRender;
exports.destroyAll = destroyAll;
exports.environment = environment;
exports.render = render;
exports.rendered = rendered;
