'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const jestMatcherUtils = require('jest-matcher-utils');
const utilities = require('./utilities.cjs');

function toContainReactComponent(node, type, props) {
  utilities.assertIsNode(node, {
    expectation: 'toContainReactComponent',
    isNot: this.isNot
  });
  utilities.assertIsType(type, {
    expectation: 'toContainReactComponent',
    isNot: this.isNot
  });
  const foundByType = node.findAll(type);
  const foundByProps = props == null ? foundByType : foundByType.filter(element => this.equals(props, utilities.getObjectSubset(element.props, props)));
  const pass = foundByProps.length > 0;
  const message = pass ? () => `${jestMatcherUtils.matcherHint('.not.toContainReactComponent')}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `Not to contain component:\n  ${jestMatcherUtils.EXPECTED_COLOR(utilities.printType(type))}\n${props ? `With props matching:\n  ${jestMatcherUtils.printExpected(props)}\n` : ''}` + `But ${foundByProps.length} matching ${utilities.printType(type)} ${foundByProps.length === 1 ? 'elements were' : 'element was'} found.\n` : () => `${`${jestMatcherUtils.matcherHint('.toContainReactComponent')}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `To contain component:\n  ${jestMatcherUtils.EXPECTED_COLOR(utilities.printType(type))}\n${props ? `With props matching:\n  ${jestMatcherUtils.printExpected(props)}\n` : ''}`}${foundByType.length === 0 ? `But no matching ${utilities.printType(type)} elements were found.\n` : `But the ${foundByType.length === 1 ? 'found element has' : 'found elements have'} the following prop differences:\n\n${utilities.diffs(foundByType, props, this.expand)}`}`;
  return {
    pass,
    message
  };
}
function toContainReactComponentTimes(node, type, times, props) {
  utilities.assertIsNode(node, {
    expectation: 'toContainReactComponentTimes',
    isNot: this.isNot
  });
  utilities.assertIsType(type, {
    expectation: 'toContainReactComponent',
    isNot: this.isNot
  });
  const foundByType = node.findAll(type);
  const foundByProps = props == null ? foundByType : foundByType.filter(element => this.equals(props, utilities.getObjectSubset(element.props, props)));
  const pass = foundByProps.length === times;
  const message = pass ? () => [`${jestMatcherUtils.matcherHint('.not.toContainReactComponentTimes')}\n`, `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}`, `Not to contain component:\n  ${jestMatcherUtils.EXPECTED_COLOR(utilities.printType(type))}`, `${times} ${utilities.pluralize('time', times)}, but it did.`].join('\n') : () => [`${jestMatcherUtils.matcherHint('.toContainReactComponentTimes')}\n`, `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}`, `To contain component:\n  ${jestMatcherUtils.EXPECTED_COLOR(utilities.printType(type))}`, `${times} ${utilities.pluralize('time', times)}, but it was found ${foundByProps.length}.`].join('\n');
  return {
    pass,
    message
  };
}

exports.toContainReactComponent = toContainReactComponent;
exports.toContainReactComponentTimes = toContainReactComponentTimes;
