'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const jestMatcherUtils = require('jest-matcher-utils');
const utilities = require('./utilities.cjs');

function toProvideReactContext(node, Context, value) {
  utilities.assertIsNode(node, {
    expectation: 'toProvideReactContext',
    isNot: this.isNot
  });
  const foundByType = node.findAll(Context.Provider);
  const foundByValue = value == null ? foundByType : foundByType.filter(element => Object.is(value, element.prop('value')));
  const pass = foundByValue.length > 0;
  const message = pass ? () => `${jestMatcherUtils.matcherHint('.not.toProvideReactContext')}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `Not to contain context provider:\n  ${jestMatcherUtils.EXPECTED_COLOR(utilities.printType(Context.Provider))}\n${value ? `With value matching:\n  ${jestMatcherUtils.printExpected(value)}\n` : ''}` + `But ${foundByValue.length} matching ${utilities.printType(Context.Provider)}${foundByValue.length === 1 ? 's were' : ' was'} found.\n` : () => `${`${jestMatcherUtils.matcherHint('.toProvideReactContext')}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `To contain context provider:\n  ${jestMatcherUtils.EXPECTED_COLOR(utilities.printType(Context.Provider))}\n${value ? `With value matching:\n  ${jestMatcherUtils.printExpected(value)}\n` : ''}`}${foundByType.length === 0 ? `But no matching ${utilities.printType(Context.Provider)}s were found.\n` : `But the ${foundByType.length === 1 ? 'found provider has' : 'found provider have'} had different values:\n\n${utilities.diffs(foundByType, {
    value
  }, this.expand)}`}`;
  return {
    pass,
    message
  };
}

exports.toProvideReactContext = toProvideReactContext;
