'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const jestMatcherUtils = require('jest-matcher-utils');
const props = require('./props.cjs');
const utilities = require('./utilities.cjs');

function toContainReactHTML(node, text) {
  utilities.assertIsNode(node, {
    expectation: 'toContainReactHTML',
    isNot: this.isNot
  });
  const nodeHTML = node.html;
  const matchIndex = nodeHTML.indexOf(text);
  const pass = matchIndex >= 0;
  const message = pass ? () => `${jestMatcherUtils.matcherHint('.not.toContainReactHTML', node.toString())}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `Not to contain HTML:\n  ${jestMatcherUtils.printExpected(text)}\n` + `But it did:\n  ${utilities.printReceivedWithHighlight(nodeHTML, matchIndex, text.length)}\n` : () => `${jestMatcherUtils.matcherHint('.not.toContainReactHTML', node.toString())}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `With HTML content:\n  ${jestMatcherUtils.printReceived(nodeHTML)}\n` + `To contain HTML:\n  ${jestMatcherUtils.printExpected(text)}\n`;
  return {
    pass,
    message
  };
}
function toHaveReactDataProps(node, data) {
  return props.toHaveReactProps.call(this, node, data);
}

exports.toContainReactHTML = toContainReactHTML;
exports.toHaveReactDataProps = toHaveReactDataProps;
