'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const jestMatcherUtils = require('jest-matcher-utils');
const utilities = require('./utilities.cjs');

function toHaveReactProps(node, props) {
  utilities.assertIsNode(node, {
    expectation: 'toHaveReactProps',
    isNot: this.isNot
  });
  if (props == null || typeof props !== 'object') {
    return {
      pass: false,
      message: () => `You passed ${props == null ? String(props) : `a ${typeof props}`} as props, but it must be an object.`
    };
  }
  const pass = this.equals(props, utilities.getObjectSubset(node.props, props));
  const message = pass ? () => `${jestMatcherUtils.matcherHint('.not.toHaveReactProps', node.toString())}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `Not to have props:\n  ${jestMatcherUtils.printExpected(props)}\n` + `Received:\n  ${jestMatcherUtils.printReceived(node.props)}\n` : () => {
    const diffString = utilities.diffPropsForNode(node, props, {
      expand: this.expand
    });
    return `${jestMatcherUtils.matcherHint('.toHaveReactProps', node.toString())}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `To have props:\n  ${jestMatcherUtils.printExpected(props)}\n` + `Received:\n  ${jestMatcherUtils.printReceived(node.props)}\n${diffString ? `Difference:\n${diffString}\n` : ''}`;
  };
  return {
    pass,
    message
  };
}

exports.toHaveReactProps = toHaveReactProps;
