import { act, create } from 'react-test-renderer';
import { createEnvironment } from '../environment.mjs';

const environment = createEnvironment({
  act,
  mount(tree) {
    const renderer = create(tree);
    return {
      renderer
    };
  },
  unmount({
    renderer
  }) {
    renderer.unmount();
  },
  update(_, create, {
    renderer
  }) {
    return createNodeFromTestInstance(renderer.root, create);
  }
});
const {
  render,
  createRender,
  rendered,
  destroyAll
} = environment;
function createNodeFromTestInstance(testInstance, create) {
  function children() {
    return testInstance.children.map(child => typeof child === 'string' ? child : createNodeFromTestInstance(child, create)) ?? [];
  }
  return create({
    props: testInstance.props,
    type: testInstance.type,
    children,
    instance: testInstance.instance
  });
}

export { createRender, destroyAll, environment, render, rendered };
