import { matcherHint, RECEIVED_COLOR, printExpected, printReceived } from 'jest-matcher-utils';
import { toHaveReactProps } from './props.mjs';
import { assertIsNode, printReceivedWithHighlight } from './utilities.mjs';

function toContainReactHTML(node, text) {
  assertIsNode(node, {
    expectation: 'toContainReactHTML',
    isNot: this.isNot
  });
  const nodeHTML = node.html;
  const matchIndex = nodeHTML.indexOf(text);
  const pass = matchIndex >= 0;
  const message = pass ? () => `${matcherHint('.not.toContainReactHTML', node.toString())}\n\n` + `Expected the React element:\n  ${RECEIVED_COLOR(node.toString())}\n` + `Not to contain HTML:\n  ${printExpected(text)}\n` + `But it did:\n  ${printReceivedWithHighlight(nodeHTML, matchIndex, text.length)}\n` : () => `${matcherHint('.not.toContainReactHTML', node.toString())}\n\n` + `Expected the React element:\n  ${RECEIVED_COLOR(node.toString())}\n` + `With HTML content:\n  ${printReceived(nodeHTML)}\n` + `To contain HTML:\n  ${printExpected(text)}\n`;
  return {
    pass,
    message
  };
}
function toHaveReactDataProps(node, data) {
  return toHaveReactProps.call(this, node, data);
}

export { toContainReactHTML, toHaveReactDataProps };
