import type { ReactElement } from 'react';
import type { Node as BaseNode, Root as RootNode, PlainObject, EmptyObject, MergeObjects } from './types.ts';
export type { BaseNode as Node, RootNode as Root };
export interface BaseNodeCreationOptions<T, Extensions extends PlainObject> {
    props: T;
    instance: any;
    type: BaseNode<T, Extensions>['type'];
    children: () => (BaseNode<unknown, Extensions> | string)[];
}
export type NodeCreationOptions<T, Extensions extends PlainObject = EmptyObject> = EmptyObject extends Extensions ? BaseNodeCreationOptions<T, Extensions> : BaseNodeCreationOptions<T, Extensions> & Extensions;
export interface EnvironmentNodeCreator<Extensions extends PlainObject> {
    <T>(options: NodeCreationOptions<T, Extensions>): BaseNode<T, Extensions>;
}
export interface EnvironmentOptions<Context, Extensions extends PlainObject = EmptyObject> {
    act<T>(action: () => T): T extends Promise<any> ? Promise<void> : void;
    mount(element: ReactElement<any>): Context;
    unmount(context: Context): void;
    update(instance: any, create: EnvironmentNodeCreator<Extensions>, context: Context): BaseNode<unknown, Extensions> | null;
}
export type AfterRenderOption<RenderOptions extends PlainObject, Context extends PlainObject, Actions extends PlainObject, Extensions extends PlainObject, Async extends boolean> = Async extends true ? {
    afterRender(wrapper: RootNode<unknown, Context, Actions, Extensions>, options: RenderOptions): PromiseLike<void>;
} : {
    afterRender?(wrapper: RootNode<unknown, Context, Actions, Extensions>, options: RenderOptions): void;
};
export type ContextOption<RenderOptions extends PlainObject = EmptyObject, ExistingContext extends PlainObject = EmptyObject, AdditionalContext extends PlainObject = EmptyObject> = AdditionalContext extends EmptyObject ? {
    context?: never;
} : {
    context(options: RenderOptions, existingContext: ExistingContext): AdditionalContext;
};
export type ActionsOption<RenderOptions extends PlainObject = EmptyObject, Context extends PlainObject = EmptyObject, ExistingActions extends PlainObject = EmptyObject, AdditionalActions extends PlainObject = EmptyObject, Extensions extends PlainObject = EmptyObject> = AdditionalActions extends EmptyObject ? {
    actions?: never;
} : {
    actions(root: Omit<RootNode<unknown, Context, any, Extensions>, 'actions'>, options: RenderOptions, existingActions: ExistingActions): AdditionalActions;
};
export interface RenderOption<RenderOptions extends PlainObject = EmptyObject, Context extends PlainObject = EmptyObject> {
    render?(element: ReactElement<any>, context: Context, options: RenderOptions): ReactElement<any>;
}
export type RenderOptionsOverrideOption<ExistingRenderOptions extends PlainObject = EmptyObject, AdditionalRenderOptions extends PlainObject = EmptyObject> = AdditionalRenderOptions extends EmptyObject ? {
    options?: never;
} : {
    options?(current: MergeObjects<ExistingRenderOptions, AdditionalRenderOptions>): Partial<AdditionalRenderOptions>;
};
export type CustomRenderOptions<RenderOptions extends PlainObject = EmptyObject, ExistingContext extends PlainObject = EmptyObject, AdditionalContext extends PlainObject = EmptyObject, ExistingActions extends PlainObject = EmptyObject, AdditionalActions extends PlainObject = EmptyObject, Extensions extends PlainObject = EmptyObject, Async extends boolean = false> = RenderOption<RenderOptions, MergeObjects<ExistingContext, AdditionalContext>> & ContextOption<RenderOptions, ExistingContext, AdditionalContext> & ActionsOption<RenderOptions, MergeObjects<ExistingContext, AdditionalContext>, ExistingActions, AdditionalActions, Extensions> & AfterRenderOption<RenderOptions, MergeObjects<ExistingContext, AdditionalContext>, MergeObjects<ExistingActions, AdditionalActions>, Extensions, Async>;
export type CustomRenderExtendOptions<ExistingRenderOptions extends PlainObject = EmptyObject, AdditionalRenderOptions extends PlainObject = EmptyObject, ExistingContext extends PlainObject = EmptyObject, AdditionalContext extends PlainObject = EmptyObject, ExistingActions extends PlainObject = EmptyObject, AdditionalActions extends PlainObject = EmptyObject, Extensions extends PlainObject = EmptyObject, Async extends boolean = false> = CustomRenderOptions<MergeObjects<ExistingRenderOptions, AdditionalRenderOptions>, ExistingContext, AdditionalContext, ExistingActions, AdditionalActions, Extensions, Async> & RenderOptionsOverrideOption<ExistingRenderOptions, AdditionalRenderOptions>;
export interface CustomRender<RenderOptions extends PlainObject, Context extends PlainObject, Actions extends PlainObject, Extensions extends PlainObject, Async extends boolean> {
    <Props>(element: ReactElement<any>, options?: RenderOptions): CustomRenderResult<Props, Context, Actions, Extensions, Async>;
    extend<AdditionalRenderOptions extends PlainObject = EmptyObject, AdditionalContext extends PlainObject = EmptyObject, AdditionalActions extends PlainObject = EmptyObject, AdditionalAsync extends boolean = false>(options: CustomRenderExtendOptions<RenderOptions, AdditionalRenderOptions, Context, AdditionalContext, Actions, AdditionalActions, Extensions, AdditionalAsync>): CustomRender<MergeObjects<RenderOptions, AdditionalRenderOptions>, MergeObjects<Context, AdditionalContext>, MergeObjects<Actions, AdditionalActions>, Extensions, AdditionalAsync extends true ? AdditionalAsync : Async>;
    hook<T>(useHook: () => T, options?: RenderOptions): Async extends true ? Promise<HookRunner<T, Context, Actions>> : HookRunner<T, Context, Actions>;
}
export interface HookRunner<HookReturn, Context extends PlainObject, Actions extends PlainObject> {
    readonly current: HookReturn;
    readonly value: HookReturn;
    readonly context: Context;
    readonly actions: Actions;
    mount(): void;
    unmount(): void;
    act<T>(action: (value: HookReturn) => T): T;
}
export type CustomRenderResult<Props, Context extends PlainObject, Actions extends PlainObject, Extensions extends PlainObject, Async extends boolean> = Async extends true ? Promise<RootNode<Props, Context, Actions, Extensions>> : RootNode<Props, Context, Actions, Extensions>;
export interface Environment<Extensions extends PlainObject = EmptyObject> {
    readonly rendered: Set<RootNode<any, any, any, Extensions>>;
    readonly render: CustomRender<EmptyObject, EmptyObject, EmptyObject, Extensions, false>;
    createRender<RenderOptions extends PlainObject = EmptyObject, Context extends PlainObject = EmptyObject, Actions extends PlainObject = EmptyObject, Async extends boolean = false>(options: CustomRenderOptions<RenderOptions, Context, Context, Actions, Actions, Extensions, Async>): CustomRender<RenderOptions, Context, Actions, Extensions, Async>;
    destroyAll(): void;
}
export declare function createEnvironment<EnvironmentContext = undefined, Extensions extends PlainObject = EmptyObject>(env: EnvironmentOptions<EnvironmentContext, Extensions>): Environment<Extensions>;
export declare function isNode<Extensions extends PlainObject = EmptyObject>(maybeNode: unknown): maybeNode is BaseNode<unknown, Extensions>;
//# sourceMappingURL=environment.d.ts.map