import type { ComponentType, Context as ReactContext } from 'react';
import type { PropsFor } from './types.ts';
import { toContainReactHTML, toHaveReactDataProps } from './matchers/dom.ts';
import { toHaveReactProps } from './matchers/props.ts';
import { toContainReactComponent, toContainReactComponentTimes } from './matchers/components.ts';
import { toContainReactText } from './matchers/text.ts';
import { toProvideReactContext } from './matchers/context.ts';
export interface CustomMatchers<R = unknown> {
    toHaveReactProps(props: Record<string, unknown>): void;
    toContainReactComponent<Type extends string | ComponentType<any>>(type: Type, props?: Partial<PropsFor<Type>>): R;
    toContainReactComponentTimes<Type extends string | ComponentType<any>>(type: Type, times: number, props?: Partial<PropsFor<Type>>): R;
    toProvideReactContext<Type>(context: ReactContext<Type>, value?: Type): R;
    toContainReactText(text: string): R;
    toContainReactHTML(text: string): R;
    toHaveReactDataProps(data: {
        [key: string]: string;
    }): R;
}
export declare const matchers: {
    toHaveReactProps: typeof toHaveReactProps;
    toContainReactComponent: typeof toContainReactComponent;
    toContainReactComponentTimes: typeof toContainReactComponentTimes;
    toProvideReactContext: typeof toProvideReactContext;
    toContainReactText: typeof toContainReactText;
    toContainReactHTML: typeof toContainReactHTML;
    toHaveReactDataProps: typeof toHaveReactDataProps;
};
//# sourceMappingURL=matchers.d.ts.map